/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.impl;

import javax.annotation.Nonnull;
import java.io.IOException;
import org.openrdf.model.Graph;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.io.GraphUnmarshaller;
import it.tidalwave.semantic.io.impl.GraphUnmarshallerImpl;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.factsheet.FactSheet;
import it.tidalwave.bluebill.factsheet.GraphProvider;
import it.tidalwave.bluebill.factsheet.semantic.io.FactSheetStatementUnmarshallerFactory;
import it.tidalwave.bluebill.factsheet.spi.FactSheetProvider;
import it.tidalwave.semantic.Type;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@ServiceProvider(service=FactSheetProvider.class) @Slf4j
public class DefaultFactSheetProvider implements FactSheetProvider
  {
    private static final Type TYPE_TAXON = new Type("http://rs.tdwg.org/dwc/2009-12-07/terms/Taxon"); // FIXME: move to a Vocabulary
    
    @Nonnull
    public FactSheet createFactSheetFor (final @Nonnull Taxon taxon)
      throws NotFoundException
      {
        try
          {
            final GraphProvider graphProvider = new DefaultGraphProvider(taxon);
            final Graph graph = graphProvider.getGraph();
            final GraphUnmarshaller unmarshaller = new GraphUnmarshallerImpl();
            unmarshaller.registerStatementUnmarshallerFactory(new FactSheetStatementUnmarshallerFactory());
            return unmarshaller.unmarshal(graph, TYPE_TAXON);
          }
        catch (IOException e)
          {
            throw new NotFoundException(e);  
          }
      }    
  }
