/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.semantic.io;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.List;
import org.openrdf.model.Literal;
import org.openrdf.model.Statement;
import org.openrdf.model.Value;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.Key;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerSupport;
import it.tidalwave.mobile.media.Media;
import static it.tidalwave.semantic.vocabulary.DublinCoreVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class FoafImageUnmarshaller extends StatementUnmarshallerSupport
  {
    private static final List<Id> PREDICATES_REFERRING_ENTITIES = Arrays.asList(ID_DC_CREATOR, ID_DC_PUBLISHER);
    private static final List<Id> PREDICATES_REFERRING_IDS = Arrays.asList(ID_DC_IDENTIFIER);

    @Override @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        final Id id = new Id(statements.get(0).getSubject().stringValue());
        Media media = new Media().with(Media.ID, id);

        for (final Statement statement : statements)
          {
            final Value object = statement.getObject();
            final Id predicate = new Id(statement.getPredicate().stringValue());
            final Key<Object> key = new Key<Object>(predicate);

            if (PREDICATES_REFERRING_ENTITIES.contains(predicate))
              {
                media = media.with(key, context.find(object));
              }
            else if (PREDICATES_REFERRING_IDS.contains(predicate))
              {
                media = media.with(key, new Id(object.stringValue()));
              }
            else if (object instanceof Literal)
              {
                media = media.with(key, object.stringValue());
              }
          }

        return media;
      }
  }
